@@ -35,9 +35,9 @@
 #include <unistd.h>
 
 
-// Date of this version: 2004-10-30
+// Date of this version: 2004-12-14
 
-const char * const program_version = "0.7";
+const char * const program_version = "0.8";
 const char * const program_year    = "2004";
 
 bool volatile interrupted = false;		// user pressed Ctrl-C
@@ -47,8 +47,8 @@ struct Bigblock
   {
   off_t ipos, opos, size;
   bool ignore;
-  Bigblock() : ipos( 0 ), opos( 0 ), size( 0 ), ignore( false ) {}
-  Bigblock( off_t i, off_t o, off_t s, bool ig = false )
+  Bigblock() : ipos( 0 ), opos( 0 ), size( 0 ), ignore( true ) {}
+  Bigblock( off_t i, off_t o, off_t s, bool ig = true )
     : ipos( i ), opos( o ), size( s ), ignore( ig ) {}
   };
 
@@ -79,7 +79,7 @@ void show_version() throw()
 void show_error( const char * msg, bool show_errno = false ) throw()
   {
   std::fprintf( stderr, "ddrescue: %s", msg );
-  if( show_errno ) std::fprintf( stderr, ": %s", strerror( errno ) );
+  if( errno && show_errno ) std::fprintf( stderr, ": %s", strerror( errno ) );
   std::fprintf( stderr, "\n" );
   }
 
@@ -96,6 +96,7 @@ void show_help( const char * program_name, int cluster, int hardbs ) throw()
   std::printf( "  -c, --cluster-size=<blocks>  hardware blocks to copy at a time [%d]\n", cluster );
   std::printf( "  -e, --max-errors=<n>         maximum number of error areas allowed\n" );
   std::printf( "  -i, --input-position=<pos>   start position in input file [0]\n" );
+  std::printf( "  -n, --no-split               do not try to split error areas\n" );
   std::printf( "  -o, --output-position=<pos>  start position in output file [ipos]\n" );
   std::printf( "  -q, --quiet                  quiet operation\n" );
   std::printf( "  -r, --max-retries=<n>        exit after given retries (-1=infinity) [0]\n" );
@@ -109,6 +110,57 @@ void show_help( const char * program_name, int cluster, int hardbs ) throw()
   }
 
 
+long long getnum( const char * ptr, size_t bs,
+                  long long min = LONG_LONG_MIN + 1,
+                  long long max = LONG_LONG_MAX ) throw()
+  {
+  errno = 0;
+  char *tail;
+  long long result = std::strtoll( ptr, &tail, 0 );
+  if( tail == ptr )
+    { show_error( "bad or missing numerical argument" ); std::exit(1); }
+
+  if( !errno )
+    {
+    int n = 0;
+    switch( *tail )
+      {
+      case 'c':
+      case ' ':
+      case  0 : break;
+      case 'b': if( bs && LONG_LONG_MAX / bs < std::llabs( result ) ) errno = ERANGE;
+                else result *= bs;
+                break;
+      case 'Y': ++n;
+      case 'Z': ++n;
+      case 'E': ++n;
+      case 'P': ++n;
+      case 'T': ++n;
+      case 'G': ++n;
+      case 'M': ++n;
+      case 'k': for( ++n; n > 0 && !errno; --n )
+                  { if( LONG_LONG_MAX / 1024 < std::llabs( result ) ) errno = ERANGE;
+                  else result *= 1024; }
+                break;
+      default: show_error( "bad suffix in numerical argument" ); std::exit(1);
+      }
+    }
+  if( !errno && ( result < min || result > max ) ) errno = ERANGE;
+  if( errno )
+    { show_error( "numerical argument out of limits" ); std::exit(1); }
+  return result;
+  }
+
+
+bool check_identical( const char * name1, const char * name2 ) throw()
+  {
+  if( std::strcmp( name1, name2 ) == 0 ) return true;
+  struct stat stat1, stat2;
+  if( stat( name1, &stat1) || stat( name2, &stat2) ) return false;
+  return ( stat1.st_ino == stat2.st_ino && stat1.st_dev == stat2.st_dev );
+  }
+
+
 const char * format_num( long long num, long long max = 999999 ) throw()
   {
   static const char * const units[8] =
@@ -162,15 +214,16 @@ void show_status( off_t ipos, off_t opos, off_t recsize, off_t errsize,
 size_t readblock( int fd, char * buf, size_t size, off_t pos ) throw()
   {
   int rest = size;
-  lseek( fd, pos, SEEK_SET );
-  while( rest > 0 )
-    {
-    errno = 0;
-    int n = read( fd, buf + size - rest, rest );
-    if( n > 0 ) rest -= n;
-    else if( n == 0 ) break;
-    else if( errno != EINTR && errno != EAGAIN ) break;
-    }
+  errno = 0;
+  if( lseek( fd, pos, SEEK_SET ) >= 0 )
+    while( rest > 0 )
+      {
+      int n = read( fd, buf + size - rest, rest );
+      if( n > 0 ) rest -= n;
+      else if( n == 0 ) break;
+      else if( errno != EINTR && errno != EAGAIN ) break;
+      errno = 0;
+      }
   return ( rest > 0 ) ? size - rest : size;
   }
 
@@ -178,22 +231,34 @@ size_t readblock( int fd, char * buf, size_t size, off_t pos ) throw()
 size_t writeblock( int fd, char * buf, size_t size, off_t pos ) throw()
   {
   int rest = size;
-  lseek( fd, pos, SEEK_SET );
-  while( rest > 0 )
-    {
-    errno = 0;
-    int n = write( fd, buf + size - rest, rest );
-    if( n > 0 ) rest -= n;
-    else if( n == 0 ) break;
-    else if( errno != EINTR && errno != EAGAIN ) break;
-    }
+  errno = 0;
+  if( lseek( fd, pos, SEEK_SET ) >= 0 )
+    while( rest > 0 )
+      {
+      int n = write( fd, buf + size - rest, rest );
+      if( n > 0 ) rest -= n;
+      else if( n == 0 ) break;
+      else if( errno != EINTR && errno != EAGAIN ) break;
+      errno = 0;
+      }
   return ( rest > 0 ) ? size - rest : size;
   }
 
 
-// Returns true if returned block is part of a bigger one at front()
+// Returns true if 'block' spans more than one hardware block.
+bool can_split_block( const Block & block, const size_t hardbs ) throw()
+  {
+  size_t size = ( block.ipos + block.size ) % hardbs;
+  if( size == 0 ) size = hardbs;
+  return ( size < block.size );
+  }
+
+
+// Splits front() and returns in 'block' its last hardware block.
+// Returns true if returned block is part of a bigger one at front().
+// Returns false if returned block is front() itself.
 bool split_block( Block & block, std::queue< Block > & badclusters,
-                  size_t hardbs ) throw()
+                  const size_t hardbs ) throw()
   {
   block = badclusters.front();
   size_t size = ( block.ipos + block.size ) % hardbs;
@@ -208,27 +273,30 @@ bool split_block( Block & block, std::queue< Block > & badclusters,
 
 
 int copyfile( Bigblock & bigblock, std::queue< Block > & badclusters,
-              std::queue< Block > & badblocks, off_t errsize, size_t cluster,
-              size_t hardbs, int ides, int odes, int max_errors,
-              int max_retries, int verbosity ) throw()
+              std::queue< Block > & badblocks, off_t errsize,
+              const size_t cluster, const size_t hardbs, const int ides,
+              const int odes, const int max_errors, const int max_retries,
+              const int verbosity, const bool nosplit ) throw()
   {
   off_t recsize = 0;
+  const int bbsize0 = badblocks.size();
   const size_t softbs = cluster * hardbs;
   char buf[softbs];
+  bool first_post;
 
   interrupted = false;
   signal( SIGINT, sighandler );
-  if( verbosity >= 0 )
-    {
-    std::printf( "Press Ctrl-C to interrupt\n" );
-    show_status( bigblock.ipos, bigblock.opos, recsize, errsize,
-                 badclusters.size(), 0, true );
-    }
+  if( verbosity >= 0 ) std::printf( "Press Ctrl-C to interrupt\n" );
 
-  // 1) Read the non-damaged part of the disk, skipping the damaged areas.
+  // Read the non-damaged part of the disk, skipping over the damaged areas.
+  first_post = true;
   if( !bigblock.ignore )
     while( !interrupted && bigblock.size != 0 )
       {
+      if( verbosity >= 0 )
+        { show_status( bigblock.ipos, bigblock.opos, recsize, errsize,
+                       badclusters.size() + badblocks.size(),
+                       "Copying data...", first_post ); first_post = false; }
       size_t size = softbs;
       if( bigblock.size >= 0 && bigblock.size < (off_t)size )
         size = bigblock.size;
@@ -242,84 +310,115 @@ int copyfile( Bigblock & bigblock, std::queue< Block > & badclusters,
         }
       else rd = readblock( ides, buf, size, bigblock.ipos );
       recsize += rd;
+      int errno1 = errno;
 
       if( rd > 0 && writeblock( odes, buf, rd, bigblock.opos ) != rd )
         { show_error( "write error", true ); return 1; }
       if( rd < size )
         {
-        if( rd == 0 && !errno ) break;		// EOF
-        else					// Read error
+        if( !errno1 )	// EOF
+          { bigblock.ipos += rd; bigblock.opos += rd; bigblock.size = 0; break; }
+        else		// Read error
           {
           badclusters.push( Block( bigblock.ipos + rd, bigblock.opos + rd, size - rd ) );
           errsize += ( size - rd );
-          if( max_errors >= 0 && (int)badclusters.size() > max_errors )
+          if( max_errors >= 0 &&
+              bbsize0 + (int)badclusters.size() > max_errors )
             { show_error( "too many errors in input file" ); return 2; }
           }
         }
       bigblock.ipos += size; bigblock.opos += size;
       if( bigblock.size > 0 ) bigblock.size -= size;
-      if( verbosity >= 0 )
-        show_status( bigblock.ipos, bigblock.opos, recsize, errsize,
-                     badclusters.size(), "Copying data..." );
       }
 
-
-  // 2) Try to read the damaged areas, splitting them into smaller pieces
-  //    and reading the non-damaged pieces, until the hardware block size
-  //    is reached, or until interrupted by the user.
   Block block( bigblock.ipos, bigblock.opos, 0 );	// Default values
-  bool split = false;
-  while( !interrupted && badclusters.size() )
-    {
-    if( split_block( block, badclusters, hardbs ) ) split = true;
-    else { badblocks.push( block ); continue; }
-    size_t rd = readblock( ides, buf, block.size, block.ipos );
-    recsize += rd;
-    errsize -= rd;
 
-    if( rd > 0 && writeblock( odes, buf, rd, block.opos ) != rd )
-      { show_error( "write error", true ); return 1; }
-    if( rd < block.size && ( rd > 0 || errno ) )	// read_error
+  // Delimit the damaged areas.
+  first_post = true;
+  if( !bigblock.ignore && nosplit )
+    for( int i = badclusters.size(); i > 0 && !interrupted; --i )
+      while( !interrupted )
+        {
+        if( !split_block( block, badclusters, hardbs ) )
+          { badblocks.push( block ); break; }
+        if( verbosity >= 0 )
+          { show_status( block.ipos, block.opos, recsize, errsize,
+                         badclusters.size() + badblocks.size(),
+                         "Delimiting error areas...", first_post );
+          first_post = false; }
+        size_t rd = readblock( ides, buf, block.size, block.ipos );
+        recsize += rd; errsize -= rd;
+        int errno1 = errno;
+
+        if( rd > 0 && writeblock( odes, buf, rd, block.opos ) != rd )
+          { show_error( "write error", true ); return 1; }
+        if( rd < block.size && ( rd > 0 || errno1 ) )	// read_error
+          {
+          badblocks.push( Block( block.ipos+rd, block.opos+rd, block.size-rd ) );
+          badclusters.push( badclusters.front() ); badclusters.pop();
+          break;
+          }
+        else errsize -= ( block.size - rd );
+        }
+
+  // Try to read the damaged areas, splitting them into smaller pieces
+  // and reading the non-damaged pieces, until the hardware block size
+  // is reached, or until interrupted by the user.
+  first_post = true;
+  if( !nosplit )
+    while( !interrupted && badclusters.size() )
       {
-      badblocks.push( Block( block.ipos+rd, block.opos+rd, block.size-rd ) );
-      badclusters.push( badclusters.front() ); badclusters.pop();
+      if( !split_block( block, badclusters, hardbs ) )
+        { badblocks.push( block ); continue; }
+      if( verbosity >= 0 )
+        { show_status( block.ipos, block.opos, recsize, errsize,
+                       badclusters.size() + badblocks.size(),
+                       "Splitting error areas...", first_post );
+        first_post = false; }
+      size_t rd = readblock( ides, buf, block.size, block.ipos );
+      recsize += rd; errsize -= rd;
+      int errno1 = errno;
+
+      if( rd > 0 && writeblock( odes, buf, rd, block.opos ) != rd )
+        { show_error( "write error", true ); return 1; }
+      if( rd < block.size && ( rd > 0 || errno1 ) )	// read_error
+        {
+        badblocks.push( Block( block.ipos+rd, block.opos+rd, block.size-rd ) );
+        badclusters.push( badclusters.front() ); badclusters.pop();
+        }
+      else errsize -= ( block.size - rd );
       }
-    else errsize -= ( block.size - rd );
-    if( verbosity >= 0 )
-      show_status( block.ipos, block.opos, recsize, errsize,
-                   badclusters.size() + badblocks.size(),
-                   "Splitting error areas..." );
-    }
 
-
-  // 3) Try to read the damaged hardware blocks until the specified number
-  //    of retries is reached, or until interrupted by the user.
+  // Try to read the damaged hardware blocks until the specified number
+  // of retries is reached, or until interrupted by the user.
   int retry = 0, counter = 0;
-  if( !split && max_retries == 0 ) max_retries = 1;
+  char msgbuf[80] = "Copying bad blocks...";
+  const int msglen = strlen( msgbuf );
+  first_post = true;
   while( !interrupted && badblocks.size() )
     {
-    char msgbuf[80];
     if( --counter <= 0 )
       {
       ++retry; counter = badblocks.size();
-      if( max_retries >= 0 && retry > max_retries ) break;
-      int i = std::snprintf( msgbuf, sizeof( msgbuf ), "Copying bad blocks..." );
-      if( max_retries != 1 && i < (int)sizeof( msgbuf ) )
-        std::snprintf( msgbuf + i, sizeof( msgbuf ) - i, " Retry %d", retry );
+      if( max_retries >= 0 && retry > max_retries &&
+          ( retry > 1 || !bbsize0 ) ) break;
+      if( max_retries < 0 || max_retries > 1 )
+        std::snprintf( msgbuf + msglen, sizeof( msgbuf ) - msglen, " Retry %d", retry );
       }
     block = badblocks.front(); badblocks.pop();
+    if( verbosity >= 0 )
+      { show_status( block.ipos, block.opos, recsize, errsize,
+                     badclusters.size() + badblocks.size() + 1,
+                     msgbuf, first_post ); first_post = false; }
     size_t rd = readblock( ides, buf, block.size, block.ipos );
-    recsize += rd;
-    errsize -= rd;
+    recsize += rd; errsize -= rd;
+    int errno1 = errno;
 
     if( rd > 0 && writeblock( odes, buf, rd, block.opos ) != rd )
       { show_error( "write error", true ); return 1; }
-    if( rd < block.size && ( rd > 0 || errno ) )	// read_error
+    if( rd < block.size && ( rd > 0 || errno1 ) )	// read_error
       badblocks.push( Block( block.ipos+rd, block.opos+rd, block.size-rd ) );
     else errsize -= ( block.size - rd );
-    if( verbosity >= 0 )
-      show_status( block.ipos, block.opos, recsize, errsize,
-                   badblocks.size(), msgbuf );
     }
 
   if( verbosity >= 0 )
@@ -333,60 +432,10 @@ int copyfile( Bigblock & bigblock, std::queue< Block > & badclusters,
   }
 
 
-long long getnum( const char * ptr, size_t bs,
-                  long long min = LONG_LONG_MIN + 1,
-                  long long max = LONG_LONG_MAX ) throw()
-  {
-  errno = 0;
-  char *tail;
-  long long result = std::strtoll( ptr, &tail, 0 );
-  if( tail == ptr )
-    { show_error( "bad or missing numerical argument" ); std::exit(1); }
-
-  if( !errno )
-    {
-    int n = 0;
-    switch( *tail )
-      {
-      case 'c':
-      case ' ':
-      case  0 : break;
-      case 'b': if( bs && LONG_LONG_MAX / bs < std::llabs( result ) ) errno = ERANGE;
-                else result *= bs;
-                break;
-      case 'Y': ++n;
-      case 'Z': ++n;
-      case 'E': ++n;
-      case 'P': ++n;
-      case 'T': ++n;
-      case 'G': ++n;
-      case 'M': ++n;
-      case 'k': for( ++n; n > 0 && !errno; --n )
-                  { if( LONG_LONG_MAX / 1024 < std::llabs( result ) ) errno = ERANGE;
-                  else result *= 1024; }
-                break;
-      default: show_error( "bad suffix in numerical argument" ); std::exit(1);
-      }
-    }
-  if( !errno && ( result < min || result > max ) ) errno = ERANGE;
-  if( errno )
-    { show_error( "numerical argument out of limits" ); std::exit(1); }
-  return result;
-  }
-
-
-bool check_identical( const char * name1, const char * name2 ) throw()
-  {
-  if( std::strcmp( name1, name2 ) == 0 ) return true;
-  struct stat stat1, stat2;
-  if( stat( name1, &stat1) || stat( name2, &stat2) ) return false;
-  return ( stat1.st_ino == stat2.st_ino && stat1.st_dev == stat2.st_dev );
-  }
-
-
 bool read_badblocks_file( const char * bbname, Bigblock & bigblock,
                           std::queue< Block > & badclusters,
-                          off_t & errsize, int max_errors ) throw()
+                          std::queue< Block > & badblocks, off_t & errsize,
+                          const int max_errors, const size_t hardbs ) throw()
   {
   if( !bbname ) return false;
   FILE *f = std::fopen( bbname, "r" );
@@ -403,10 +452,14 @@ bool read_badblocks_file( const char * bbname, Bigblock & bigblock,
     show_error( buf ); std::exit(1);
     }
   bigblock.ipos = ipos; bigblock.opos = opos; bigblock.size = size;
+  bigblock.ignore = ( bigblock.size == 0 );
   }
 
+  while( badclusters.size() ) badclusters.pop();
+  while( badblocks.size() ) badblocks.pop();
   errsize = 0;
-  while( max_errors < 0 || (int)badclusters.size() <= max_errors )
+  int errors;
+  for( errors = 0; max_errors < 0 || errors <= max_errors; ++errors )
     {
     long long ipos, opos;
     int size;
@@ -419,12 +472,14 @@ bool read_badblocks_file( const char * bbname, Bigblock & bigblock,
                      bbname, badclusters.size() + 2 );
       show_error( buf ); std::exit(1);
       }
-    badclusters.push( Block( ipos, opos, size ) );
     errsize += size;
+    Block block( ipos, opos, size );
+    if( can_split_block( block, hardbs ) ) badclusters.push( block );
+    else badblocks.push( block );
     }
 
   std::fclose( f );
-  if( max_errors >= 0 && (int)badclusters.size() > max_errors )
+  if( max_errors >= 0 && errors > max_errors )
     { show_error( "too many blocks in badblocks file" ); std::exit(1); }
   return true;
   }
@@ -492,6 +547,7 @@ void set_bigblock( Bigblock & bigblock, off_t ipos, off_t opos, off_t max_size,
     if( ipos >= 0 ) bigblock.ipos = ipos; else bigblock.ipos = 0;
     if( opos >= 0 ) bigblock.opos = opos; else bigblock.opos = bigblock.ipos;
     bigblock.size = max_size;
+    bigblock.ignore = ( bigblock.size == 0 );
     }
   else if( ipos >= 0 || opos >= 0 || max_size > 0 )
     {
@@ -516,6 +572,7 @@ int main( int argc, char * argv[] ) throw()
   off_t ipos = -1, opos = -1, max_size = -1;
   size_t cluster = 128, hardbs = 512;
   int max_errors = -1, max_retries = 0, o_trunc = 0, verbosity = 0;
+  bool nosplit = false;
 
   while( true )
     {
@@ -523,20 +580,21 @@ int main( int argc, char * argv[] ) throw()
       {
       {"block-size", required_argument, 0, 'b'},
       {"cluster-size", required_argument, 0, 'c'},
-      {"max-errors", required_argument, 0, 'e'},
       {"help", no_argument, 0, 'h'},
       {"input-position", required_argument, 0, 'i'},
-      {"output-position", required_argument, 0, 'o'},
-      {"quiet", no_argument, 0, 'q'},
+      {"max-errors", required_argument, 0, 'e'},
       {"max-retries", required_argument, 0, 'r'},
       {"max-size", required_argument, 0, 's'},
+      {"no-split", no_argument, 0, 'n'},
+      {"output-position", required_argument, 0, 'o'},
+      {"quiet", no_argument, 0, 'q'},
       {"truncate", no_argument, 0, 't'},
       {"verbose", no_argument, 0, 'v'},
       {"version", no_argument, 0, 'V'},
       {0, 0, 0, 0}
       };
 
-    int c = getopt_long( argc, argv, "b:c:e:hi:o:qr:s:tvV", long_options, 0 );
+    int c = getopt_long( argc, argv, "b:c:e:hi:no:qr:s:tvV", long_options, 0 );
     if( c == -1 ) break;		// all options processed
 
     switch( c )
@@ -546,6 +604,7 @@ int main( int argc, char * argv[] ) throw()
       case 'e': max_errors = getnum( optarg, hardbs, 0, INT_MAX ); break;
       case 'h': show_help( argv[0], cluster, hardbs ); return 0;
       case 'i': ipos = getnum( optarg, hardbs, 0 ); break;
+      case 'n': nosplit = true; break;
       case 'o': opos = getnum( optarg, hardbs, 0 ); break;
       case 'q': verbosity = -1; break;
       case 'r': max_retries = getnum( optarg, hardbs, -1, INT_MAX ); break;
@@ -582,34 +641,44 @@ int main( int argc, char * argv[] ) throw()
   Bigblock bigblock;
   std::queue< Block > badclusters, badblocks;
   off_t errsize = 0;
-  bool bbmode = read_badblocks_file( bbname, bigblock, badclusters, errsize, max_errors );
+  bool bbmode = read_badblocks_file( bbname, bigblock, badclusters, badblocks,
+                                     errsize, max_errors, hardbs );
   set_bigblock( bigblock, ipos, opos, max_size, isize, bbmode );
 
   if( cluster < 1 ) cluster = 1;
   if( hardbs < 1 ) hardbs = 1;
 
+  if( bigblock.ignore && ( nosplit || !badclusters.size() ) && !badblocks.size() )
+    { if( verbosity >= 0 ) { show_error( "Nothing to do" ); } return 0; }
+
   if( verbosity > 0 )
     {
-    if( bigblock.size != 0 && !bigblock.ignore )
+    if( !bigblock.ignore )
       {
-      std::printf( "\nabout to copy %sBytes from %s to %s\n",
-                   (bigblock.size >= 0) ? format_num( bigblock.size ) : "an infinitude of ",
+      std::printf( "\nAbout to copy %sBytes from %s to %s",
+                   (bigblock.size >= 0) ?
+                     format_num( bigblock.size ) : "an undefined number of ",
                    iname, oname );
-      std::printf( "starting positions: infile = %sB,", format_num( bigblock.ipos ) );
-      std::printf( "  outfile = %sB\n", format_num( bigblock.opos ) );
-      std::printf( "copy block size: %d hard blocks\n", cluster );
+      std::printf( "\n    Starting positions: infile = %sB,  outfile = %sB",
+                   format_num( bigblock.ipos ), format_num( bigblock.opos ) );
+      std::printf( "\n    Copy block size: %d hard blocks", cluster );
       }
-    if( badclusters.size() )
-      std::printf( "\nabout to copy %d bad blocks from %s to %s\n\n",
+    if( !nosplit && badclusters.size() )
+      std::printf( "\nAbout to split and copy %d error areas from %s to %s",
                    badclusters.size(), iname, oname );
-    std::printf( "hard block size: %d bytes\n", hardbs );
+    if( badblocks.size() )
+      std::printf( "\nAbout to copy %d bad blocks from %s to %s",
+                   badblocks.size(), iname, oname );
+    std::printf( "\nHard block size: %d bytes\n", hardbs );
     if( max_errors >= 0 ) std::printf( "Max_errors: %d    ", max_errors );
     if( max_retries >= 0 ) std::printf( "Max_retries: %d    ", max_retries );
+    std::printf( "Split: %s    ", !nosplit ? "yes" : "no" );
     std::printf( "Truncate: %s\n\n", o_trunc ? "yes" : "no" );
     }
 
   int retval = copyfile( bigblock, badclusters, badblocks, errsize, cluster,
-                         hardbs, ides, odes, max_errors, max_retries, verbosity );
+                         hardbs, ides, odes, max_errors, max_retries,
+                         verbosity, nosplit );
   if( !retval )
     retval = write_badblocks_file( bbname, bigblock, badclusters, badblocks );
   return retval;
